#include <dirent.h>
#include <qapp.h>
#include <qlayout.h>
#include <qstring.h>
#include <qmsgbox.h>
#include <qobjcoll.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qpixmap.h>
#include <qfiledlg.h>
#include <errno.h>
#include "helpers/helpers.h"
#include "options.h"
#include "mwidget.h"
#include "SelectMixer.h"
#include <qtooltip.h>
#include <kprocess.h>
#include "config.h"

#include <kapp.h>

#include <kmainwindow.h>
#include <kconfig.h>
#include "mainwidget.h"

extern KApplication*MainApp;
QToolTipGroup * TipGroup;

// Constructor of the main widget
// break into more parts next release
MyWidget::MyWidget( )
    : KMainWindow()
{
    statusBar()->message("XqMixer "+QString(VERSION));
    // two layouts which take the  elements
    //    int width_counter = 0;
    //statusbar =  new KStatusBar(this);
    //CHECK_PTR(statusbar);
    //setStatusBar(statusbar);   

    //statusbar->insertItem(version_string,1);
    /*    KConfig * k = kapp->getConfig();
    k->setGroup("common");
    int w,h;
    QString ws,hs;
    ws = k->readEntry("width","0");
    hs = k->readEntry("height","0");
    w = ws.toInt();
    h = hs.toInt();
    
    if (w && h)
      resize(w,h);
    else
      adjustSize();
    */
}

void MyWidget::SetDevice(const char*device)
{
    QString dev;
    
    if (device)
	dev = device;
    else
	dev = "/dev/mixer";

    /*
      QString title;
      title = caption();
      title+=" - ";
      title+=device;
      setCaption(title);
    */
    KMenuBar* menubar = new KMenuBar(this);    
    CHECK_PTR(menubar);
    TipGroup = new QToolTipGroup(this);
    CHECK_PTR(TipGroup);

    setCaption("XWindows Mixer");
    mainmixer = new MixerWidget(this,dev);
    CHECK_PTR(mainmixer);

    file = new QPopupMenu;
    CHECK_PTR(file);

    file->insertItem("&Open other mixerdevice",this,SLOT(open_mixer()),CTRL+Key_O);
    file->insertItem("&Load mixer from file",mainmixer,SLOT(read_mixer()),CTRL+Key_L);
    file->insertItem("&Save mixer to file",mainmixer,SLOT(save_mixer()),CTRL+Key_S);
    file->insertSeparator();
    file->insertItem("&Close Window",this,SLOT(windowClose()),CTRL+Key_W);
    file->insertItem("E&xit",this,SLOT(raus_hier()),CTRL+Key_Q);

    menubar->insertItem("&File",file);
    opt = new QPopupMenu;
    CHECK_PTR(opt);

    opt->setCheckable(TRUE);
    disp_id = opt->insertItem("ValueDisplays on",this,SLOT(toggle_value_display()),CTRL+Key_D);
    last_act = 0;
    opt->insertSeparator();
    los = opt->insertItem("Load on startup",this,SLOT(load_on_startup()));
    lol = opt->insertItem("Load only Layout",this,SLOT(load_only_layout()));
    lov = opt->insertItem("Load only Values",this,SLOT(load_only_values()));
    opt->insertSeparator();
    soe = opt->insertItem("Save all on exit",this,SLOT(save_on_exit()));
    opt->insertItem("Save Options",this,SLOT(save_opts()));
    //int optid = 
    menubar->insertItem("&Options",opt);
    menubar->insertSeparator();
    QString about_string = "";//version_string;
    about_string+=" by Rajko \"Alwin\" Albrecht\n";
    about_string+="e-mail: rajko.albrecht@informatik.tu-chemnitz.de";
    KPopupMenu*f = helpMenu();/*true,
                                about_string);*/
    CHECK_PTR(f);

    menubar->insertItem("&Help",(QPopupMenu*)f);

    opt->insertItem("Save Values", mainmixer,SLOT(save_vals()));

    opt->setItemChecked(soe,mainmixer->Options->get_save_on_exit());
    opt->setItemChecked(los,mainmixer->Options->get_load_on_start());
    opt->setItemEnabled(lol,mainmixer->Options->get_load_on_start());
    opt->setItemChecked(lol,mainmixer->Options->get_load_only_layout());
    opt->setItemEnabled(lov,mainmixer->Options->get_load_on_start());

    if (!mainmixer->Options->get_load_only_layout()) 
	opt->setItemChecked(lov,mainmixer->Options->get_load_only_vals());
    else {
	opt->setItemChecked(lov,FALSE);
	mainmixer->Options->set_load_only_vals(0);
    }
    //menubar->show();
    //setMenu(menubar);
    //menubar->enableFloating(TRUE);
    //mainmixer->setGeometry( 2, 2, 366, 186 );
    mainmixer->setMinimumSize( 370, 190 );
    mainmixer->setMaximumSize( 32767, 32767 );
    setCentralWidget(mainmixer);
    mainmixer->setGeometry( 2, 2, 366, 186 );

    QString info;
    info = "xqmixer "+QString(VERSION);//version_string;
    if (mainmixer->get_dev_name()) {
	info+=" (";
	info+=mainmixer->get_dev_name();
	info+=")";
    }
    statusBar()->message(info);
  

    connect(TipGroup,SIGNAL(showTip(const QString&)),this,SLOT(changeStatusbar(const QString&)));
    connect(TipGroup,SIGNAL(removeTip()),this,SLOT(deleteStatusText()));
    KConfig * k = KGlobal::config();
    k->setGroup("common");
    int w,h;
    QString ws,hs;
    ws = k->readEntry("width","0");
    hs = k->readEntry("height","0");
    w = ws.toInt();
    h = hs.toInt();
    
    if (w && h)
      resize(w,h);
    else
      adjustSize();
    info = mainmixer->get_dev_name();
    int pos = info.find(' ');
    if (pos > 0) {
	info = info.right(info.length()-pos);
    }
    setCaption("XWindows Mixer: "+info+" ("+dev+")");
}

void MyWidget::toggle_value_display()
{
    mainmixer->toggle_value_display(last_act);
    last_act = (last_act)?0:1;
    if (last_act)
	opt->changeItem("ValueDisplays off",disp_id);
    else
	opt->changeItem("ValueDisplays on",disp_id);
}

MyWidget::~MyWidget()
{

}

void MyWidget::save_on_exit()
{
    if (mainmixer->Options->get_save_on_exit()) {
	mainmixer->Options->set_save_on_exit(0);
	opt->setItemChecked(soe,FALSE);
    }
    else {
	mainmixer->Options->set_save_on_exit(1);
	opt->setItemChecked(soe,TRUE);
    }
}

void MyWidget::load_on_startup()
{
    if (mainmixer->Options->get_load_on_start()) {
	mainmixer->Options->set_load_on_start(0);
	opt->setItemChecked(los,FALSE);
	opt->setItemEnabled(lol,FALSE);
	opt->setItemEnabled(lov,FALSE);
    }
    else {
	mainmixer->Options->set_load_on_start(1);
	opt->setItemChecked(los,TRUE);
	opt->setItemEnabled(lol,TRUE);
	opt->setItemEnabled(lov,TRUE);
    }
}

void MyWidget::load_only_layout()
{
    if (mainmixer->Options->get_load_only_layout()) {
	opt->setItemChecked(lol,FALSE);
	mainmixer->Options->set_load_only_layout(0);
    }
    else {
	opt->setItemChecked(lol,TRUE);
	mainmixer->Options->set_load_only_layout(1);
	opt->setItemChecked(lov,FALSE);
	mainmixer->Options->set_load_only_vals(0);
    }
}

void MyWidget::load_only_values()
{
    if (mainmixer->Options->get_load_only_vals()) {
	opt->setItemChecked(lov,FALSE);
	mainmixer->Options->set_load_only_vals(0);
    }
    else {
	opt->setItemChecked(lov,TRUE);
	mainmixer->Options->set_load_only_vals(1);
	opt->setItemChecked(lol,FALSE);
	mainmixer->Options->set_load_only_layout(0);
    }
}

void MyWidget::save_opts()
{
    if (mainmixer && mainmixer->Options)
	mainmixer->Options->save_options();
}

void MyWidget::changeStatusbar(const QString&text)
{
    KStatusBar*status = statusBar();
    if (status) {
	status->message(text);
    }
}

void MyWidget::deleteStatusText()
{
    KStatusBar*status = statusBar();
    
    if (!status)
	return;
    QString info;
    info = "xqmixer "+QString(VERSION);/*version_string;*/
    if (mainmixer->get_dev_name()) {
	info+=" (";
	info+=mainmixer->get_dev_name();
	info+=")";
    }
    statusBar()->message(info);
}

void MyWidget::windowClose()
{
    close();
}

void MyWidget::raus_hier()
{

    //    kapp->quit();
    //    saveOptions();
    // close the first window, the list makes the next one the first again.
    // This ensures that queryClose() is called on each window to ask for closing
#if 0
    KMainWindow* w;
    if(memberList){
	for(w=memberList->first(); w; w=memberList->first()){
	    // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
	    // the window and the application stay open.
	    if(!w->close())
		break;
	}
    }
#endif
    close();
}

int sel(const dirent*dir)
{
    if (!dir)
	return 0;
    if (strstr(dir->d_name,"mixer"))
	return 1;
    return 0;
}

void MyWidget::open_mixer()
{
    struct dirent **namelist;
    int n;

    QStrList name_list;

    n = scandir("/dev", &namelist, &sel, alphasort);
    if (n < 0) {
	perror("scandir");
    } else {
	while(n--) {
	    QString a= "/dev/";
	    a+=namelist[n]->d_name;
	    name_list.append(a);
	}
    }
    if (name_list.count() == 0)
	return;
    CSelectMixer mix_select(this);
    if (!mix_select.setList(&name_list))
	return;
    n = mix_select.exec();
    if (n == 0)
	return;
    QString name = mix_select.get_selection();
    MyWidget*w = new MyWidget();
    w->SetDevice(name);
    w->show();
}

bool MyWidget::queryClose()
{
    return true;
}

bool MyWidget::queryExit()
{
    if (mainmixer->Options) {
	if (mainmixer->Options->get_save_on_exit()) {
	    save_opts();
	    mainmixer->save_vals();
	}
    }
    KConfig*k = KGlobal::config();
    k->setGroup("common");
    int w,h;
    w = width();
    h = height();
    QString s;
    s.setNum(w);
    k->writeEntry("width",s);
    s.setNum(h);
    k->writeEntry("height",s);
    return true;
}
